#include <StaticConstants.au3>
#include <Misc.au3>
#include <Crypt.au3>
#include <File.au3>
#include "Zip.au3"
#include-once

Global $__AIUM_BASEURL, $__AIUM_PROJECTID
Global $__AIUM_MAJOR, $__AIUM_MINOR, $__AIUM_PATCH
Global $__AIUM_PREF

Func _AIUM_Init($sBaseURL, $sProjectID)
	$sProjectID = StringLower($sProjectID)
	If StringRight($sBaseURL, 1) <> "/" Then $sBaseURL &= "/"

	$__AIUM_BASEURL = $sBaseURL
	$__AIUM_PROJECTID = $sProjectID
	$__AIUM_PREF = ObjCreate("Scripting.Dictionary")
	$__AIUM_PREF("string.update_window_title") = "Update"
	$__AIUM_PREF("string.update_not_available") = "Kein Update verfgbar."
	$__AIUM_PREF("string.update_available") = "Ein Update auf Version %s ist verfgbar. Jetzt herunterladen?"
	$__AIUM_PREF("string.update_file_incorrect") = "Die Datei konnte nicht heruntergeladen werden oder sie ist fehlerhaft."
	$__AIUM_PREF("string.update_file_correct") = "Das Update wurde erfolgreich heruntergeladen und wird nun verarbeitet."
	$__AIUM_PREF("string.update_is_downloading") = "Das Update wird jetzt heruntergeladen. Bitte warten Sie..."
	$__AIUM_PREF("boolean.update_not_available_silent") = False
EndFunc   ;==>_AIUM_Init

Func _AIUM_ThisVersion($iMajor, $iMinor, $iPatch)
	$__AIUM_MAJOR = Int($iMajor)
	$__AIUM_MINOR = Int($iMinor)
	$__AIUM_PATCH = Int($iPatch)
EndFunc   ;==>_AIUM_ThisVersion

Func _AIUM_UpdateCheck($funcUpdate)
	$sRequestURL = "http://" & $__AIUM_BASEURL & "api/info.php?id=" & $__AIUM_PROJECTID
	$sProjectInfo = BinaryToString(InetRead($sRequestURL, 1))

	$iMajor = __AIUM_IniStrRead($sProjectInfo, "major")
	$iMinor = __AIUM_IniStrRead($sProjectInfo, "minor")
	$iPatch = __AIUM_IniStrRead($sProjectInfo, "patch")

	$sLocalVersion = $__AIUM_MAJOR & "." & $__AIUM_MINOR & "." & $__AIUM_PATCH
	$sRemoteVersion = $iMajor & "." & $iMinor & "." & $iPatch

	$iCompare = _VersionCompare($sLocalVersion, $sRemoteVersion)
	Switch $iCompare
		Case 0
			If Not $__AIUM_PREF("boolean.update_not_available_silent") Then
				MsgBox(64, $__AIUM_PREF("string.update_window_title"), $__AIUM_PREF("string.update_not_available"))
			EndIf
			Return
		Case 1
			If Not $__AIUM_PREF("boolean.update_not_available_silent") Then
				MsgBox(64, $__AIUM_PREF("string.update_window_title"), "Interessant! Die lokale Version ist aktueller. Sind Sie mein Entwickler?")
			EndIf
			Return
		Case -1
			$iAns = MsgBox(64 + 4, $__AIUM_PREF("string.update_window_title"), StringFormat($__AIUM_PREF("string.update_available"), $sRemoteVersion))
			If $iAns = 7 Then Return
	EndSwitch

	$sSavename = @TempDir & "\" & __AIUM_IniStrRead($sProjectInfo, "basename")
	__AIUM_Download(__AIUM_IniStrRead($sProjectInfo, "url"), $sSavename, __AIUM_IniStrRead($sProjectInfo, "size"))

	;verify file by md5
	$sLocalMD5 = StringTrimLeft(_Crypt_HashFile($sSavename, $CALG_MD5), 2)
	$sRemoteMD5 = __AIUM_IniStrRead($sProjectInfo, "md5")
	If $sLocalMD5 <> $sRemoteMD5 Then
		MsgBox(16, $__AIUM_PREF("string.update_window_title"), $__AIUM_PREF("string.update_file_incorrect"))
		Return
	EndIf
	MsgBox(64, $__AIUM_PREF("string.update_window_title"), $__AIUM_PREF("string.update_file_correct"))
	$funcUpdate($sSavename)
EndFunc   ;==>_AIUM_UpdateCheck

Func _AIUM_SetPref($sKey, $sValue)
	$__AIUM_PREF(StringLower($sKey)) = $sValue
EndFunc

Func __AIUM_IniStrRead($sIniStr, $sKey)
	$sPattern = "(?m)^" & $sKey & "\=(.*?)$"
	$aGrep = StringRegExp($sIniStr, $sPattern, 3)
	If IsArray($aGrep) Then Return $aGrep[0]
EndFunc

Func __AIUM_Download($sDownloadURL, $sSavename, $iSize)
	$hWnd = GUICreate($__AIUM_PREF("string.update_window_title"), 400, 150)
	$cInfo = GUICtrlCreateLabel($__AIUM_PREF("string.update_ist_downloading"), 10, 10, 380, 20)
	$cProgress = GUICtrlCreateProgress(50, 50, 300, 30)
	$cProgressNum = GUICtrlCreateLabel("", 50, 90, 300, 20, $SS_CENTER + $SS_CENTERIMAGE)
	GUISetState()

	$hDownload = InetGet($sDownloadURL, $sSavename, 1, 1)
	Do
		$iYet = InetGetInfo($hDownload, 0)
		GUICtrlSetData($cProgressNum, StringFormat("%d%% (%d / %d)", $iYet / $iSize * 100, $iYet, $iSize))
		GUICtrlSetData($cProgress, $iYet / $iSize * 100)
	Until InetGetInfo($hDownload, 2)
	$iYet = InetGetInfo($hDownload, 0)
	ConsoleWrite(InetGetInfo($hDownload, 4) & @CRLF)
	GUICtrlSetData($cProgressNum, StringFormat("%d%% (%d / %d)", $iYet / $iSize * 100, $iYet, $iSize))
	GUICtrlSetData($cProgress, $iYet / $iSize * 100)

	InetClose($hDownload)
	GUIDelete($hWnd)
EndFunc

Func _AIUM_PreDefined_SelfReplace($sFile)
	$sBatchName = "update.bat"
	$sCMD  = 'DEL "' & @ScriptFullPath & '"' & @CRLF
	$sCMD &= 'COPY "' & $sFile & '" "' & @ScriptFullPath & '"' & @CRLF
	$sCMD &= 'DEL "' & $sBatchName & '"'
	FileWrite($sBatchName, $sCMD)
	Run($sBatchName)
	Exit
EndFunc

Func _AIUM_PreDefined_ZipSelfReplace($sFile)
	DirCreate(@ScriptDir & "\update.temp")
	_Zip_UnzipAll($sFile, @ScriptDir & "\update.temp")
	$aFiles = _FileListToArray(@ScriptDir, "*", 1)
	$aFolders = _FileListToArray(@ScriptDir, "*", 2)
	$sBatchName = "update.bat"
	$sCMD = ""
;~ 	For $i = 1 To $aFiles[0]
;~ 		If $aFiles[$i] = $sBatchName Then ContinueLoop
;~ 		$sCMD &= 'DEL "' & $aFiles[$i] & '"' & @CRLF
;~ 	Next
;~ 	For $i = 1 To $aFolders[0]
;~ 		If $aFolders[$i] = "update.temp" Then ContinueLoop
;~ 		$sCMD &= 'RMDIR /S /Q"' & $aFiles[$i] & '"' & @CRLF
;~ 	Next
	$sCMD &= 'XCOPY "' & @ScriptDir & '\update.temp\*" "' & @ScriptDir & '" /s /i /y' & @CRLF
	$sCMD &= 'RMDIR /S /Q "update.temp"' & @CRLF
	$sCMD &= 'DEL "' & $sBatchName & '"'
	FileWrite($sBatchName, $sCMD)
	Run($sBatchName)
	Exit
EndFunc